/* Filename: outgauge.h
   Compiler: wxDevCpp Version 7.3.1.3 (MINGW32)
   Project:  OutGauge Bus
   Original Author:   Vladimir Kadlec http://www.fi.muni.cz/~xkadlec
   
   Revision: Serial Interface and Packet Update revisions  - Pete Willard
   Version: LFS.Z28.01
 *
 * Serial Modification for microcontroller support:  Pete Willard
 * Now sends commands out serial ports to an external Microcontroller 
 * serial bus.
 *
 * A lot of the original code was scrapped, Sorry Vladimir.
 * I just didn't want to figure out WINSOCK UDP code by myself.  This is
 * therefore, a very modified derivitive of the original GI.EXE
*/ 

/*
 The goal of this set project is to read the outgauge packets and
 generate the appropriate Serial string to be send to the Outgauge Bus.
 Listeners on the serial bus respond to preset commands and provide an
 expandable dashboard bus for Gear indicators, RPM gauges and lights, 
 warnings, etc... each controlled by a stand-alone micro-controller. 
*/
 
/*

ORIGINAL: Gear Indicator is free software; you can redistribute it and/or 
modify it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Gear Indicator is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Gear Indicator; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// Under Project Options, you can include an ICON file (*.ico) evsn
// for Console programs.  One has been provided.

#include <windows.h>          // Includes winsock2.h for network calls
// NOTE: Besure to include Dev-Cpp/lib/libwsock32.a in Linker settings
//       of Project Options-->Parameters-->Linker Settings
//
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
//
#include <signal.h>           // For CTRL-C handling
//
#include "outgauge.h"         // Scawen's OUTGAUGE packet format

#include "rs232.h"            // Serial Communication routines
#include "log.h"              // debugging and error control
#include "config.h"           // external config file routines

//-----------------------------------------------------------------------------
//global variable, unfortunately, the signal handler is the reason...
Config *CONFIG;
int *IN_SKT;



//-----------------------------------------------------------------------------
void error_exit() {
     fprintf (stderr, "Press [Enter] to exit program.\n");
     getc (stdin);
     exit (-1);
}
//-----------------------------------------------------------------------------
void print_name() {
    printf ("OutGauge Bus: an OutGauge serial client for LFS\n");
    printf ("Coder  : Pete Willard\n\n");
    printf ("Version: 20/12/10 Updated for S2 Patch 0.5Z.28\n");    
    printf ("Based on a Gear Indicator project by Vladimir Kadlec\n");
    printf ("         & Nick Aronson (see LFS Forum)\n\n");


}

//-----------------------------------------------------------------------------
void signalHandler (int sig) {
   switch (sig){
       case SIGINT:  // SIGINT is the CRTL-C sequence flag
          RS232_clear (CONFIG);
//          closesocket (*IN_SKT);  //not sure if this is needed.
//          WSACleanup();
          exit (0);
          break;
       default:
          printf("Unknown signal caught=%d\n",sig);
          break;
    };
}



//-----[M A I N ]--------------------------------------------------------------
int main (int argc, char *argv[]) {
//-----------------------------------------------------------------------------
struct hostent *localhost;
int in_skt;                              // winsock UDP port number                                                                                                        
struct sockaddr_in server_addr;          // winsock destination address for API                                       

OutGaugePack og_packet;                  // UDP Packet definition for payload



WORD wVersionRequested = MAKEWORD(1,1); // Standard API call - Winsock Version
WSADATA wsadata;                        // Standard API structure - Winsock2.dll 
static Config conf;

       CONFIG = &conf;
       IN_SKT = &in_skt;
       
// Startup routines       
    
       signal (SIGINT, signalHandler);  // Set CTRL/C Interrupt Handler
    
       print_name();                    // Print startup screen 
       
       init_config (&conf);             // open config and load values
       
       if (!read_config ("config.txt", &conf)) {
          error ("Read_config\n");
          error_exit();
          }
       print_config (&conf);                   

	   if (!RS232_init (conf.com,conf.speed)) {    // Open RS232 port
          error ("Can't access COM port.\n");
          error_exit();
	      }
	      	      
	      // Command Line option to reset displays
          // Command is: -- blank-and-exit
               
    if ((argc > 1) && (strcasecmp (argv[1], "--blank-and-exit") == 0)) {
         RS232_clear (CONFIG);
         exit (0);
         }

          
          // Setup Windows Network API Socket Handler
    
    if (WSAStartup(wVersionRequested, &wsadata) != 0) {
         error ("Init socket.\n");
         error_exit();
         }
	
         localhost = gethostbyname (conf.ip);
         
         if (localhost == NULL) {
	        error ("Can't resolve IP address: %s\n", conf.ip);
            WSACleanup();
		    error_exit ();
	     }
	     
      // UDP Packet Setup using standard Microsoft API calls
	  in_skt = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	  
	  if (in_skt == INVALID_SOCKET) {
		 error ("Create socket.\n");
         WSACleanup();
		 error_exit ();
	  }
	
	  server_addr.sin_family = AF_INET;
      server_addr.sin_port = htons (conf.port);
      memcpy(&(server_addr.sin_addr), localhost->h_addr, localhost->h_length);

	//zadny pripojeni v tehle chvili, pracujem s UDP, je to jen prirazeni
	//adresy
    //-------------------------------------------------------------------------
	// Set up handler to receive UDP packet and deliver to outgauge variables
    //-------------------------------------------------------------------------
    printf ("Going to bind...\n");
	if (bind (in_skt, (struct sockaddr*)&server_addr,
				sizeof(server_addr)) == SOCKET_ERROR) {
		error ("Bind()\n");
        WSACleanup();
		error_exit ();
    }
    
	printf ("Listening...\n");
    printf ("Hit Ctrl-C to exit\n");
    
    //-------------------------------------------------------------------------
    // Debug Routine:  Show what is being received
    // and receive packet then process result
    // Handler runs until CLOSE SOCKET command occurs or the program is 
	// interrupted with CTRL-C
    // This is all using standard API calls to WINSOCK
    //-------------------------------------------------------------------------
if ((conf.debug) != 0) {
  printf (" Outgauge Packet - gear indicator debugging:\n");
  while (1) {
   if (recv (in_skt,(void*) &og_packet, sizeof (og_packet), 0) == SOCKET_ERROR) 
   {
      warning ("Problem receiving OutGauge packet.\n");
   }
   printf (" [RAW Gear %d\n] ", og_packet.Gear);
   printf (" [RAW RPM  %d\n] ", (int)og_packet.RPM);	    
   fflush (stdout);
          
   RS232_display (&conf, &og_packet);
  }
}
    //-------------------------------------------------------------------------
    
    //-------------------------------------------------------------------------   
    // SAME AS ABOVE WITHOUT DEBUG
    // Recieve Packet, process result
    // Handler runs until CLOSE SOCKET command occurs or the program is 
	// interrupted with CTRL-C
    // This is all using standard API calls to WINSOCK
    //------------------------------------------------------------------------- 
while (1) {
  if (recv (in_skt,(void*) &og_packet, sizeof (og_packet), 0) == SOCKET_ERROR) 
  {
    warning ("Problem receiving OutGauge packet.\n");
  }
           
   // Send result of packet read to com port
   RS232_display (&conf, &og_packet);

}
    //------------------------------------------------------------------------- 


    // should not get here, but just in case... die gracefully.
	closesocket (in_skt);
	
	printf ("Bye\n");
    WSACleanup();
    getc (stdin);
	return (1);
}
